# -*- coding: utf-8 -*-
"""s3 class"""

__author__  = "Adrien DELLE CAVE"
__license__ = """
    Copyright (C) 2018  fjord-technologies

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import botocore
import logging
import pycurl

from mimetools import Message

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


LOG = logging.getLogger('xtrargon.classes.s3')


class XtrargonBotoNullEventHooks(botocore.hooks.BaseEventHooks):
    def _register(self, event_name, handler, unique_id=None, unique_id_uses_count=False):
        return

    def emit_until_response(self, event_name, **kwargs):
        return (None, None)


class XtrargonS3Command(object):
    def __init__(self, client = None, session = None, config = None):
        if not client and not session:
            session = botocore.session.Session(include_builtin_handlers = None,
                                               event_hooks = XtrargonBotoNullEventHooks())

        if client:
            self.client = client
        else:
            self.client = session.create_client('s3', config = config)

        self.config = self.client._client_config

    def _prepare(self, pc, op, params = None, body = None):
        if params is None:
            params = {}

        op_model = self.client._service_model.operation_model(op)

        request_context = {
            'client_region': self.client.meta.region_name,
            'client_config': self.client.meta.config,
            'has_streaming_input': op_model.has_streaming_input,
            'auth_type': op_model.auth_type,
        }

        self.req_dict = self.client._convert_to_request_dict(
                            params,
                            op_model,
                            context=request_context)

        if body is not None:
            self.req_dict['body'] = body

        self.request = botocore.awsrequest.create_request_object(self.req_dict)
        self.client._request_signer.sign(op, self.request)

        pc.setopt(pycurl.URL, str(self.request.url))
        pc.setopt(pycurl.CONNECTTIMEOUT, self.config.connect_timeout)
        pc.setopt(pycurl.CUSTOMREQUEST, str(self.request.method))
        pc.setopt(pycurl.HEADER, 1)
        pc.setopt(pycurl.FOLLOWLOCATION, 1)
        pc.setopt(pycurl.LOW_SPEED_TIME, 5)
        pc.setopt(pycurl.LOW_SPEED_LIMIT, 1024)
        pc.setopt(pycurl.NOSIGNAL, 1)
        pc.setopt(pycurl.TIMEOUT, self.config.read_timeout)
        pc.setopt(pycurl.VERBOSE, 0)
        pc.setopt(pycurl.WRITEFUNCTION, lambda x: None)

        if self.request.method == 'HEAD':
            pc.setopt(pycurl.NOBODY, 1)
        elif self.req_dict['body']:
            pc.setopt(pycurl.POSTFIELDS, self.req_dict['body'])

        headers = ['Connection: keep-alive',
                   'Content-Type: application/octet-stream']

        for x in self.request.headers.items():
            headers.append(str(": ".join(x)))

        pc.setopt(pycurl.HTTPHEADER, headers)

    def _parse_headers(self, raw_headers):
        raw_headers.seek(0)
        raw_headers.readline()
        return Message(raw_headers)

    def run(self, op, params = None, body = None, parse_headers = False):
        r             = {'headers': None,
                         'code':    None}
        pc            = None
        raw_headers   = None
        self.request  = None
        self.req_dict = None

        try:
            pc = pycurl.Curl()
            self._prepare(pc, op, params, body)

            if parse_headers:
                raw_headers = StringIO()
                pc.setopt(pycurl.HEADERFUNCTION, raw_headers.write)

            pc.perform()
            r['code'] = pc.getinfo(pc.HTTP_CODE)

            if parse_headers:
                r['headers'] = self._parse_headers(raw_headers)

            if r['code'] >= 400:
                LOG.error("return code: %r", r['code'])
                raise botocore.exceptions.ClientError({'Error': {'Code':    str(r['code']),
                                                                 'Message': ''},
                                                       'ResponseMetadata': {}},
                                                      op)
        except pycurl.error:
            raise
        finally:
            if raw_headers:
                raw_headers.close()

            if pc:
                pc.close()

            self.request  = None
            self.req_dict = None

        return r

    def __call__(self, op, params = None, body = None, parse_headers = False):
        return self.run(op, params, body, parse_headers)
